/*
 * XdccDownloader - Result.java - Copyright © 2013 David Roden
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.pterodactylus.xdcc.ui.stdin;

import static com.google.common.collect.FluentIterable.from;
import static java.util.regex.Pattern.compile;

import java.util.Collection;
import java.util.Comparator;
import java.util.function.Predicate;
import java.util.regex.Pattern;

import net.pterodactylus.xdcc.core.Core;
import net.pterodactylus.xdcc.data.Bot;
import net.pterodactylus.xdcc.data.Download;
import net.pterodactylus.xdcc.data.Pack;

import com.google.common.base.Function;
import com.google.common.collect.ComparisonChain;

/**
 * Container for result information.
 *
 * @author <a href="mailto:bombe@pterodactylus.net">David ‘Bombe’ Roden</a>
 */
public class Result implements Comparable<Result> {

	private static Predicate<String> matches(String regex) {
		Pattern pattern = compile(regex);
		return new Predicate<String>() {
			@Override
			public boolean test(String input) {
				return pattern.matcher(input).find();
			}
		};
	}

	private static Comparator<Result> preferredComparator(Predicate<String> preferredName, Function<Result, String> stringExtractor) {
		return new Comparator<Result>() {
			@Override
			public int compare(Result leftResult, Result rightResult) {
				boolean leftStringMatches = preferredName.test(stringExtractor.apply(leftResult));
				boolean rightStringMatches = preferredName.test(stringExtractor.apply(rightResult));
				if (leftStringMatches && !rightStringMatches) {
					return -1;
				} else if (rightStringMatches && !leftStringMatches) {
					return 1;
				}
				return 0;
			}
		};
	}

	/** {@link Comparator} for {@link Result}s that sorts archives to the back of the list. */
	private static final Comparator<Result> packArchiveComparator = preferredComparator(matches("(rar|tar|zip|tar\\.(gz|bz2|lzma)|7z)$").negate(), (result) -> result.pack().name());

	/** {@link Comparator} for bot nicknames. */
	private static final Comparator<Result> botNameComparator =
			sortEuropeanBotsToTheFront().thenComparing(sortAmericanBotsToTheBack()).thenComparing(sortPassiveBotsToTheBack());

	private static Comparator<Result> sortEuropeanBotsToTheFront() {
		return preferredComparator(matches("(?i)[^\\w]EUR?[^\\w]"), (result) -> result.bot().name());
	}

	private static Comparator<Result> sortAmericanBotsToTheBack() {
		return preferredComparator(matches("(?i)[^\\w]USA?[^\\w]").negate(), (result) -> result.bot().name());
	}

	private static Comparator<Result> sortPassiveBotsToTheBack() {
		return preferredComparator(matches("[-|]P[-|]").negate(), (result) -> result.bot().name());
	}

	private static final Comparator<Result> packNameComparator = sortRepacksFirst().thenComparing(sortPacksAlphabetically());

	private static Comparator<Result> sortRepacksFirst() {
		return preferredComparator(matches("\\.(PROPER|REPACK)\\."), (result) -> result.pack().name());
	}

	private static Comparator<Result> sortPacksAlphabetically() {
		return new Comparator<Result>() {
			@Override
			public int compare(Result leftResult, Result rightResult) {
				return leftResult.pack().name().compareToIgnoreCase(rightResult.pack().name());
			}
		};
	}

	/** Comparator that sorts bots with running downloads to the back of the list. */
	private final Comparator<Result> botsWithRunningTransfersComparator = new Comparator<Result>() {
		@Override
		public int compare(Result leftResult, Result rightResult) {
			Collection<Bot> botsWithTransfers = from(core.downloads()).transform(new Function<Download, Bot>() {
				@Override
				public Bot apply(Download download) {
					return download.bot();
				}
			}).toSet();
			boolean leftDownloading = botsWithTransfers.contains(leftResult.bot());
			boolean rightDownloading = botsWithTransfers.contains(rightResult.bot());
			if (leftDownloading && !rightDownloading) {
				return 1;
			}
			if (!leftDownloading && rightDownloading) {
				return -1;
			}
			return 0;
		}
	};

	/** The core. */
	private final Core core;

	/** The bot carrying the pack. */
	private final Bot bot;

	/** The pack. */
	private final Pack pack;

	/**
	 * Creates a new result.
	 *
	 * @param core
	 * 		The core
	 * @param bot
	 * 		The bot carrying the pack
	 * @param pack
	 * 		The pack of the result
	 */
	Result(Core core, Bot bot, Pack pack) {
		this.core = core;
		this.bot = bot;
		this.pack = pack;
	}

	//
	// ACCESSORS
	//

	/**
	 * Returns the bot carrying the pack.
	 *
	 * @return The bot carrying the pack
	 */
	public Bot bot() {
		return bot;
	}

	/**
	 * Returns the pack.
	 *
	 * @return The pack
	 */
	public Pack pack() {
		return pack;
	}

	//
	// COMPARABLE METHODS
	//

	@Override
	public int compareTo(Result result) {
		return ComparisonChain.start()
				.compare(this, result, botsWithRunningTransfersComparator)
				.compare(this, result, packArchiveComparator)
				.compare(this, result, botNameComparator)
				.compare(this, result, packNameComparator).result();
	}

}
