/*
 * XdccDownloader - State.java - Copyright © 2013 David Roden
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.pterodactylus.xdcc.ui.stdin;

import java.util.ArrayList;
import java.util.List;

import net.pterodactylus.irc.Connection;
import net.pterodactylus.xdcc.data.Download;

import com.google.common.collect.Lists;

/**
 * Container for the current state of the command reader.
 *
 * @author <a href="mailto:bombe@pterodactylus.net">David ‘Bombe’ Roden</a>
 */
public class State {

	/** The last connections displayed. */
	private final List<Connection> lastConnections;

	/** The last results displayed. */
	private final List<Result> lastResults;

	/** The last downloads displayed. */
	private final List<Download> lastDownloads;

	private final List<Download> lastFailedDownloads;

	/** Creates a new empty state. */
	public State() {
		this(Lists.<Connection>newArrayList(), Lists.<Result>newArrayList(), Lists.<Download>newArrayList(), new ArrayList<>());
	}

	/**
	 * Creates a new state.
	 *
	 * @param lastConnections
	 * 		The last connections
	 * @param lastResults
	 * 		The last results
	 * @param lastDownloads
	 * @param lastFailedDownloads
	 * 		The last failed downloads shown
	 */
	State(List<Connection> lastConnections, List<Result> lastResults, List<Download> lastDownloads, List<Download> lastFailedDownloads) {
		this.lastConnections = lastConnections;
		this.lastResults = lastResults;
		this.lastDownloads = lastDownloads;
		this.lastFailedDownloads = lastFailedDownloads;
	}

	//
	// ACCESSORS
	//

	/**
	 * Returns the last connections displayed.
	 *
	 * @return The last connections displayed
	 */
	public List<Connection> getLastConnections() {
		return lastConnections;
	}

	/**
	 * Returns the last results displayed.
	 *
	 * @return The last results displayed
	 */
	public List<Result> getLastResults() {
		return lastResults;
	}

	/**
	 * Returns the last downloads displayed.
	 *
	 * @return The last downloads displayed
	 */
	public List<Download> getLastDownloads() {
		return lastDownloads;
	}

	public List<Download> getLastFailedDownloads() {
		return lastFailedDownloads;
	}

	//
	// MUTATORS
	//

	/**
	 * Returns a new state with the given last connections and the last downloads
	 * and results of this state.
	 *
	 * @param lastConnections
	 * 		The new last connections displayed
	 * @return The new state
	 */
	public State setLastConnections(List<Connection> lastConnections) {
		return new State(lastConnections, lastResults, lastDownloads, lastFailedDownloads);
	}

	/**
	 * Returns a new state with the given last results and the last downloads and
	 * connections of this state.
	 *
	 * @param lastResults
	 * 		The new last results displayed
	 * @return The new state
	 */
	public State setLastResults(List<Result> lastResults) {
		return new State(lastConnections, lastResults, lastDownloads, lastFailedDownloads);
	}

	/**
	 * Returns a new state with the given last downloads and the last connections
	 * and results of this state.
	 *
	 * @param lastDownloads
	 * 		The new last downloads displayed
	 * @return The new state
	 */
	public State setLastDownloads(List<Download> lastDownloads) {
		return new State(lastConnections, lastResults, lastDownloads, lastFailedDownloads);
	}

	public State setLastFailedDownloads(List<Download> lastFailedDownloads) {
		return new State(lastConnections, lastResults, lastDownloads, lastFailedDownloads);
	}

}
